@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'ostruct'
 
 module YARD
@@ -250,7 +251,8 @@ module YARD
     class MacroDirective < Directive
       def call
         raise TagFormatError if tag.name.nil? && tag.text.to_s.empty?
-        unless macro_data = find_or_create
+        macro_data = find_or_create
+        unless macro_data
           warn
           return
         end
@@ -267,8 +269,8 @@ module YARD
 
       def attach?
         new? && # must have data or there is nothing to attach
-        class_method? || # always attach to class methods
-        (tag.types && tag.types.include?('attach'))
+          class_method? || # always attach to class methods
+          (tag.types && tag.types.include?('attach'))
       end
 
       def class_method?
@@ -283,7 +285,7 @@ module YARD
       def expand(macro_data)
         return if attach? && class_method?
         return if !anonymous? && new? &&
-          (!handler || handler.statement.source.empty?)
+                  (!handler || handler.statement.source.empty?)
         call_params = []
         caller_method = nil
         full_source = ''
@@ -299,16 +301,20 @@ module YARD
       def find_or_create
         if new? || attach?
           if handler && attach?
-            obj = object ? object :
-              P("#{handler.namespace}.#{handler.caller_method}")
+            if object && object.is_a?(CodeObjects::NamespaceObject)
+              log.warn "Attaching macros to non-methods is unsupported, ignoring: " \
+                       "#{object.path} (#{handler.parser.file}:#{handler.statement.line})"
+              obj = nil
+            else
+              obj = object ? object :
+                P("#{handler.namespace}.#{handler.caller_method}")
+            end
           else
             obj = nil
           end
-          if anonymous? # anonymous macro
-            return tag.text || ""
-          else
-            macro = CodeObjects::MacroObject.create(tag.name, tag.text, obj)
-          end
+
+          return tag.text || "" if anonymous? # anonymous macro
+          macro = CodeObjects::MacroObject.create(tag.name, tag.text, obj)
         else
           macro = CodeObjects::MacroObject.find(tag.name)
         end
@@ -318,8 +324,8 @@ module YARD
 
       def warn
         if object && handler
-          log.warn "Invalid/missing macro name for " +
-            "#{object.path} (#{handler.parser.file}:#{handler.statement.line})"
+          log.warn "Invalid/missing macro name for " \
+                   "#{object.path} (#{handler.parser.file}:#{handler.statement.line})"
         end
       end
     end
@@ -332,6 +338,10 @@ module YARD
     #
     # To define an attribute method, see {tag:!attribute}
     #
+    # @note This directive should only be used if there is no explicit
+    #   declaration for the method in any source files (i.e., the method
+    #   is declared dynamically via meta-programming). In all other cases, add
+    #   documentation to the method definition itself.
     # @note For backwards compatibility support, you do not need to indent
     #   the method's docstring text. If a +@!method+ directive is seen with
     #   no indented block, the entire docstring is used as the new method's
@@ -388,7 +398,9 @@ module YARD
         handler = parser.handler
         object = parser.object
         self.parser = parser.class.new(parser.library)
+        parser.state.inside_directive = true
         parser.parse(tag.text, object, handler)
+        parser.state.inside_directive = false
       end
 
       def create_object
@@ -406,6 +418,10 @@ module YARD
         obj.docstring = Docstring.new!(parser.text, parser.tags, obj,
           parser.raw_text, parser.reference)
         handler.register_module_function(obj)
+        old_obj = parser.object
+        parser.object = obj
+        parser.post_process
+        parser.object = old_obj
         obj
       end
     end
@@ -417,8 +433,12 @@ module YARD
     # containing this directive does not need to be attached to any source, but
     # if it is, that source code will be used as the method's source.
     #
-    # To define an regular method, see {tag:!method}
+    # To define a regular method, see {tag:!method}
     #
+    # @note This directive should only be used if there is no explicit +attr_*+
+    #   declaration for the attribute in any source files (i.e., the attribute
+    #   is declared dynamically via meta-programming). In all other cases, add
+    #   documentation to the attribute declaration itself.
     # @note For backwards compatibility support, you do not need to indent
     #   the attribute's docstring text. If an +@!attribute+ directive is seen with
     #   no indented block, the entire docstring is used as the new attribute's
@@ -461,11 +481,9 @@ module YARD
         clean_name = object.name.to_s.sub(/=$/, '')
         attrs = object.namespace.attributes[object.scope]
         attrs[clean_name] ||= SymbolHash[:read => nil, :write => nil]
-        if readable?
-          attrs[clean_name][:read] = object
-        end
+        attrs[clean_name][:read] = object if readable?
         if writable?
-          if object.name.to_s[-1,1] == '='
+          if object.name.to_s[-1, 1] == '='
             writer = object
             writer.parameters = [['value', nil]]
           else
@@ -477,6 +495,7 @@ module YARD
             writer.source = object.source
             writer.group = object.group
             writer.parameters = [['value', nil]]
+            writer.docstring = object.base_docstring
             handler.register_file_info(writer)
           end
           attrs[clean_name][:write] = writer
@@ -585,6 +604,8 @@ module YARD
         if %w(public protected private).include?(tag.text)
           if object.is_a?(CodeObjects::Base)
             object.visibility = tag.text.to_sym
+          elsif handler && !parser.state.inside_directive
+            handler.visibility = tag.text.to_sym
           else
             parser.state.visibility = tag.text.to_sym
           end
@@ -592,4 +613,4 @@ module YARD
       end
     end
   end
-end
\ No newline at end of file
+end
