@@ -236,8 +236,8 @@ eof
       expect(File).to receive(:directory?).with('gem2.gem').and_return(false)
       expect(File).to receive(:exist?).with('gem1.gem').and_return(false)
       expect(File).to receive(:exist?).with('gem2.gem').and_return(false)
-      expect(@diff).to receive(:open).with('http://rubygems.org/downloads/gem1.gem')
-      expect(@diff).to receive(:open).with('http://rubygems.org/downloads/gem2.gem')
+      expect(OpenURI).to receive(:open_uri).with('http://rubygems.org/downloads/gem1.gem')
+      expect(OpenURI).to receive(:open_uri).with('http://rubygems.org/downloads/gem2.gem')
       allow(FileUtils).to receive(:mkdir_p)
       allow(Gem::Package).to receive(:open)
       allow(FileUtils).to receive(:rm_rf)
