@@ -183,6 +183,24 @@ RSpec.describe YARD::CLI::Yardoc do
       expect(Registry).not_to receive(:save)
       @yardoc.run(arg)
     end
+
+    # This example relies on processing Yard's own documentation, and with
+    # --fail-on-warning option on, will raise a SystemExit error if building
+    # that documentation produces any one warning.
+    #
+    # Unless handled, it will cause immediate and abnormal process exit,
+    # without running remaining tests, and with non-successful exit status.
+    #
+    # While suppressing exceptions is generally a bad practice and against this
+    # project's style guide, here it is well advocated,
+    # hence Lint/HandleExceptions cop is disabled.
+    should_accept('--fail-on-warning') do |arg|
+      expect(YARD).to receive(:parse)
+      begin
+        @yardoc.run(arg)
+      rescue SystemExit # rubocop:disable Lint/HandleExceptions
+      end
+    end
   end
 
   describe "Output options" do
@@ -200,10 +218,10 @@ RSpec.describe YARD::CLI::Yardoc do
     end
 
     it "aliases --main to the --readme flag" do
-      readme = File.join(File.dirname(__FILE__), '..', '..', 'README.md')
-
-      @yardoc.parse_arguments('--main', readme)
-      expect(@yardoc.options.readme).to eq CodeObjects::ExtraFileObject.new(readme, '')
+      Dir.chdir(File.join(File.dirname(__FILE__), '..', '..')) do
+        @yardoc.parse_arguments('--main', 'README.md')
+        expect(@yardoc.options.readme).to eq CodeObjects::ExtraFileObject.new('README.md', '')
+      end
     end
 
     it "selects a markup provider when --markup-provider or -mp is set" do
@@ -294,7 +312,7 @@ RSpec.describe YARD::CLI::Yardoc do
       end
 
       it "allows from:to syntax" do
-        expect(FileUtils).to receive(:cp_r).with('foo', 'doc/bar')
+        expect(FileUtils).to receive(:cp_r).with(%r{foo(\/\.)?}, 'doc/bar')
         @yardoc.run(*%w(--asset foo:bar))
         expect(@yardoc.assets).to eq('foo' => 'bar')
       end
@@ -602,15 +620,25 @@ RSpec.describe YARD::CLI::Yardoc do
     end
 
     it "warns if extra file is not found" do
-      expect(log).to receive(:warn).with(/Could not find extra file: UNKNOWN/)
+      expect(log).to receive(:warn).with(/Could not find file: UNKNOWN/)
       @yardoc.parse_arguments(*%w(- UNKNOWN))
     end
 
     it "warns if readme file is not found" do
-      expect(log).to receive(:warn).with(/Could not find readme file: UNKNOWN/)
+      expect(log).to receive(:warn).with(/Could not find file: UNKNOWN/)
       @yardoc.parse_arguments(*%w(-r UNKNOWN))
     end
 
+    it "warns on absolute paths in extra files" do
+      expect(log).to receive(:warn).with(%r{Invalid file: /path/to/file})
+      @yardoc.parse_arguments(*%w(- /path/to/file))
+    end
+
+    it "warns on absolute paths in readme" do
+      expect(log).to receive(:warn).with(%r{Invalid file: /path/to/file})
+      @yardoc.parse_arguments(*%w(-r /path/to/file))
+    end
+
     it "uses first file as readme if no readme is specified when using --one-file" do
       expect(Dir).to receive(:glob).with('README{,*[^~]}').and_return []
       expect(Dir).to receive(:glob).with('lib/*.rb').and_return(['lib/foo.rb'])
@@ -619,6 +647,12 @@ RSpec.describe YARD::CLI::Yardoc do
       expect(@yardoc.options.readme).to eq CodeObjects::ExtraFileObject.new('lib/foo.rb', '')
     end
 
+    it "uses no readme if files is empty and no readme is specified when using --one-file" do
+      expect(Dir).to receive(:glob).with('README{,*[^~]}').and_return []
+      @yardoc.parse_arguments '--one-file', ''
+      expect(@yardoc.options.readme).to be_nil
+    end
+
     it "uses readme it exists when using --one-file" do
       expect(Dir).to receive(:glob).with('README{,*[^~]}').and_return ['README']
       expect(File).to receive(:read).with('README').and_return('')
@@ -626,6 +660,34 @@ RSpec.describe YARD::CLI::Yardoc do
       expect(@yardoc.options.readme).to eq CodeObjects::ExtraFileObject.new('README', '')
     end
 
+    it "selects readme with no file extension over readme with file extension" do
+      expect(Dir).to receive(:glob).with('README{,*[^~]}').and_return ['README.md', 'README-DEV', 'README']
+      expect(File).to receive(:read).with('README').and_return('')
+      @yardoc.parse_arguments
+      expect(@yardoc.options.readme).to eq CodeObjects::ExtraFileObject.new('README', '')
+    end
+
+    it "selects readme with no suffix over readme with hyphenated suffix" do
+      expect(Dir).to receive(:glob).with('README{,*[^~]}').and_return ['README-fr.md', 'README.long-extension', 'README-de.md']
+      expect(File).to receive(:read).with('README.long-extension').and_return('')
+      @yardoc.parse_arguments
+      expect(@yardoc.options.readme).to eq CodeObjects::ExtraFileObject.new('README.long-extension', '')
+    end
+
+    it "selects readme with no suffix over readme with dotted suffix" do
+      expect(Dir).to receive(:glob).with('README{,*[^~]}').and_return ['README.fr.md', 'README.md', 'README.de.md']
+      expect(File).to receive(:read).with('README.md').and_return('')
+      @yardoc.parse_arguments
+      expect(@yardoc.options.readme).to eq CodeObjects::ExtraFileObject.new('README.md', '')
+    end
+
+    it "selects first readme from lexically sorted list" do
+      expect(Dir).to receive(:glob).with('README{,*[^~]}').and_return ['README-fr.md', 'README-de.md']
+      expect(File).to receive(:read).with('README-de.md').and_return('')
+      @yardoc.parse_arguments
+      expect(@yardoc.options.readme).to eq CodeObjects::ExtraFileObject.new('README-de.md', '')
+    end
+
     it "does not allow US-ASCII charset when using --one-file" do
       ienc = Encoding.default_internal
       eenc = Encoding.default_external
@@ -670,7 +732,7 @@ RSpec.describe YARD::CLI::Yardoc do
       @yardoc.parse_arguments('--tag', 'foo:Title of Foo')
     end
 
-    it "accepts --tag without title (and default to captialized tag name)" do
+    it "accepts --tag without title (and default to capitalized tag name)" do
       expect(Tags::Library).to receive(:define_tag).with('Foo', :foo, nil)
       @yardoc.parse_arguments('--tag', 'foo')
     end
@@ -816,5 +878,19 @@ RSpec.describe YARD::CLI::Yardoc do
       expect(Registry).not_to receive(:lock_for_writing)
       @yardoc.run('--no-save')
     end
+
+    context "with --fail-on-warning" do
+      it "exits with error status code if a warning occurs" do
+        allow(log).to receive(:warned).and_return(true)
+        expect { @yardoc.run("--fail-on-warning") }.to raise_error(SystemExit) do |error|
+          expect(error).not_to be_success
+        end
+      end
+
+      it "does not exit if a warning does not occur" do
+        allow(log).to receive(:warned).and_return(false)
+        expect { @yardoc.run("--fail-on-warning") }.not_to raise_error
+      end
+    end
   end
 end
